#encoding: UTF-8

=begin
/***************************************************************************
 *   Copyright © 2014, Michael Uplawski <michael.uplawski@uplawski.eu>     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
=end

require_relative 'logging'

# An Action is executed on a user's request.
# It has a name, an associated closure and hotkey.
class Action
  include Logging

  class ActionError < StandardError
  end

  attr_accessor :name, :key, :proc, :global, :hidden

  def to_s
    "[#<" << classname << ':' << hash << '@name="' << name << '", @key="' << key << '">'
  end


  def initialize(options = {}, &b)
    init_logger(STDOUT, Logger::INFO)

    @name = options[:name]
    @key = options[:key]
    @proc = b if b
    @global = options[:global]
    @hidden = options[:hidden]
  end

  def call(*args)
    unless @proc
      raise ActionError.new((@name ? '' : 'Unnamed ') << 'action' << (@name ? (' ' << @name) : '') << ' called before a command was defined')
    end
    @proc.call(*args) 
  end
end
