#encoding: UTF-8
=begin
/***************************************************************************
 *   Copyright (c) 2011, Michael Uplawski <michael.uplawski@uplawski.eu>   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

=end
require_relative '../color_output'

#The BusyIndicator will show an "Animation" for the time of its execution.
#The main program will continue to run in its own thread and should stop
#the BusyIndicator, once that a process of longer duration has terminated. 
class BusyIndicator
	attr_writer :width
	# Defines a busy_indicator of a width of 'width' characters.
	# If 'start' is true, the text-animation is run immediately.
	def initialize(start = true, width = nil)
		@width = width && width >= 3 ? width : 3
		@thr = busy_indicator(@width) if start
	end
	
	# Starts the text-animation, returns the thread.
	def run() 
		@thr = busy_indicator(@width)
	end

	# Stops the text-animation, terminates the thread.
	# If comment is not null, it will be displayed in the end.
	def stop(comment)
		@thr.terminate
		@thr.join
		print ("\b" * @width)
		print ("%+#{@width}s\n" %comment)
	end
	private
	def busy_indicator(width)
		tobj = Thread.new() do
			print "working ... "
			loop do 
				# %w"OOO ooo ___ ooo".each do |s|
				# %w"000 OOO UUU VVV YYY TTT 777 >>> === --- ooo".each do |s|
				%w"0OU OUV UVY VYT YT7 T7> 7>= >=- =-o -o0 o0O".each do |s|
					print "%+#{width}s" %s
					sleep 0.1
					print ("\b" * width)
				end
			end
		end
	end
end
# =======================
# Example
# You can run this file directly from a command-line, 
# like this
# user@mchine$: ruby busy_indicator.rb
#
# The program below will then start four (4) 
# BusyIndicators and stop them again, one by one.  
# The width of the "animation" is variated.
# In the loop, the BusyIndicator is started upon
# its creation, the other two instances are first
# created then 'run'.
if($0 == __FILE__)
	width = 20
	2.times do
		puts "I am busy, pse wait..."
		thr = BusyIndicator.new(true, width)
		sleep 3
		thr.stop("Okay")
	end
        bi = BusyIndicator.new(false)
	bi.run
	sleep 3
	bi.stop("Stopped")
	bi.width = 8
	bi.run
	sleep 3
	bi.stop("stopped again")
end

