#encoding: UTF-8

=begin
/***************************************************************************
 *   Copyright ©2016-2016, Michael Uplawski <michael.uplawski@uplawski.eu> *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
=end

require_relative 'logging'
require_relative 'row'
require_relative 'column'

# Objects of this class represent cells in a spreadsheet table

class Cell 
  @@DEF_HEIGHT=1
  @@DEF_TYPE=Integer
  @@split_pattern = nil

  self.extend(Logging)
  @@log = self.init_logger

  def initialize(row = nil, col = nil, value = nil)
    @log = @@log
    @row = row if row
    @col = col if col
    @type_class = @@DEF_TYPE
    @value = value
    split_value
    set_limits
    @row.resize
    @log.debug("cell.initialize, lines is #{@lines}, value is #{value}, ideal_width is #{@ideal_width}")
  end

  def to_cell()
    self
  end

  def line(num = nil)
    num && num < @lines.length ? @lines[num].to_s : ' ' if @lines && !@lines.empty?
  end

  def to_s
    object_id.to_s << "{" << @row.number.to_s << ":" << @col.number.to_s << ", " << @lines.to_s << ", " << @ideal_height.to_s << ", " << @ideal_width.to_s << " }"
  end

  def resize
    @@split_pattern = nil
    split_value
    set_limits
    @row.resize
  end

  def value=(value)
    @@split_pattern = nil
    @value = value
    split_value
    @log.debug('after split_value, lines is ' <<  @lines.to_s)
  end

  def col
    @col.number
  end

  def row
    @row.number
  end

  attr_reader :ideal_height, :ideal_width, :value

  private

  def split_value()
    # The regex may be used in many cells. Define on class-level.
    # And...
    # This looks complicated because it is.
    #
    # until 18 february 2017 
    # @@split_pattern = Regexp.new('[\p{P}\p{M}]?\b.{1,%i}\b[\p{P}\p{M}]?' %(@col.width)) if !@@split_pattern
    # since 18 february 2017
    @@split_pattern = Regexp.new('\b(?:.{1,%i}(?:[\b\s\p{P}]?))' %(@col.width - 1)) if !@@split_pattern

    @lines = @value.to_s.scan(@@split_pattern) # .collect {|match| match.strip}
    @lines.reject! {|l| l.empty?}
    set_limits
    if @value.to_s.strip.length > @lines.join.strip.length
      # @lines.insert(0, '...' )
      if(! @lines.empty?)
        @lines[@lines.length - 1] = '...'  
      else
        @lines.insert(0, '...' )
      end
    end
    @row.resize
  end

  def set_limits
    if @lines && !@lines.empty?
      @ideal_height = @lines.length
      @ideal_width = @lines.max {|l1, l2 | l1.length <=> l2.length}.length if @lines && !@lines.empty?
    else
      @ideal_width ||= @col.width 
      @ideal_height ||= @@DEF_HEIGHT
      @height ||= @ideal_height
    end
  end
end
