#encoding: UTF-8

=begin
/***************************************************************************
 *   Copyright ©2016-2016, Michael Uplawski <michael.uplawski@uplawski.eu> *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
=end


require_relative 'logging'

# Objects of this class represent columns in a spreadsheet table

class Column
  self.extend(Logging)
  @@log = self.init_logger
  @@DEF_WIDTH=15
  @@col_width=@@DEF_WIDTH
  @@columns = Array.new
  def initialize(number = nil)
    @log = @@log
    @number = number
    @cells = Array.new
    @width = @@col_width
    @@columns << self
  end

  def add(cell)
    n_cell = nil
    if cell.respond_to?(:to_cell)
      n_cell = cell
    elsif cell.respond_to?(:to_i)
      n_cell = Cell.new( cell, @number) 
    else
      msg = 'For a new cell, a row-number must be specified'
      @log.error(yellow(msg))
      raise StandardError(msg)
    end
    @cells.insert(n_cell.row, n_cell) 
    @cells.compact!
    set_limits
  end

  def each(&b)
    @cells.each do |c|
      yield(c) 
    end
  end

  def self::col_width
    @@col_width
  end

  def self::col_width=(w)
    @@col_width = w
    @log.debug("self::col_width=(#{w}), calling resize")

    @@columns.each {|col| col.resize(w)}
  end

  def resize(width)
    @width = width
    @log.debug('calling resize')
    @cells.each {|cell| cell.resize}
  end

  def to_s
    '<' << self.class.name.dup << ':' << object_id.to_s << "{number=%s width=%s cells.length=}>" %[@number, @cells.length]
  end

  attr_reader :number, :width
  alias :<< :add

  private

  def set_limits
    @ideal_width = @cells.max{|c1, c2| c1.ideal_width <=> c2.ideal_width}.ideal_width 
    @ideal_width ||= @@DEF_WIDTH
    # @width ||= @ideal_width
  end

end
