#encoding: UTF-8

=begin
/***************************************************************************
 *   Copyright © 2014-2017, Michael Uplawski <michael.uplawski@uplawski.eu>*
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
=end

require 'io/wait'
require 'io/console'
require_relative 'action'
require_relative 'logging'
require_relative 'user_input'

# Shows options that the user can choose from.
# Options may be associated with sub-menus.

class Menu 
  # include Logging
  self.extend(Logging)
  @@log = self.init_logger(STDOUT)

  attr_accessor :key, :name

  class MenuError < StandardError
  end

  @@global_elements = []
  @@hidden_elements = []

  def initialize(options = {})
    @log = @@log
    @elements = [] 
    if(options && !options.respond_to?(:to_hash) )
      raise MenuError.new('Arguments to menu.new must be hash-pairs')
    end
    @name = options[:name] if options[:name]
    @key = options[:key] if options[:key]
  end

  def call(*args)
    show()
  end

  def add(element, pos = nil)
    if(element.respond_to?(:call))
      if(pos)
        @elements.insert(pos, element)
      else
        @elements << element 
      end
      # make available the same element in any sub-menus.
      if(element.respond_to?(:global) && element.global)
        @@global_elements << element
      end
      # hidden elements
      if(element.respond_to?(:hidden) && element.hidden)
        @@hidden_elements << element
      end
    else
      raise MenuError("{#element.to_s} cannot be used as action or sub-menu: missing method 'call()'}")
    end
  end


  private
  def show()
    @elements.each do |ele|
      name = ele.name.downcase
      key = ele.key.downcase			
      print name.dup << " (#{key}) * " if !@@hidden_elements.include?(ele)
    end
    puts
    ele = nil
    until ele
      key = "%c" %wait_for_user	
=begin
      if "\e" == key
        break
      end
=end
      ele = @elements.detect{ |e| key == e.key  }
      ele ||= @@global_elements.detect{ |e| key == e.key  }
    end
    ele.call if ele
  end

  public
  alias :add_action :add
  alias :add_sub_menu :add
  alias :add_menu :add
end
