#encoding: UTF-8

=begin
/***************************************************************************
 *   Copyright ©2016-2016, Michael Uplawski <michael.uplawski@uplawski.eu> *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
=end

require_relative 'cell'
require_relative 'logging'

#  Objects of this class represent rows in a spreadsheet table
class Row
  @@DEF_HEIGHT=2
  self.extend(Logging)
  @@log = self.init_logger(STDOUT)
  def initialize(number = nil)
    @log = @@log
    @number = number
    @cells = Array.new
    @height = 1
  end

  def add(cell)
    n_cell = nil
    if cell.respond_to?(:to_cell)
      n_cell = cell
    elsif cell.respond_to?(:to_i)
      n_cell = Cell.new(@number, cell) 
    else
      msg = 'For a new cell, a colum-number must be specified'
      @log.error(red(msg))
      raise StandardError(msg)
    end
    @cells.insert(n_cell.col, n_cell) 
    @cells.compact!
    resize()
  end

  def each(&b)
    @cells.each do |c|
      yield(c) 
    end
  end

  def resize()
    if(@cells && ! @cells.empty? )
      if(@cells.length == 1 )
        @ideal_height = @cells[0].ideal_height
      else
        @ideal_height = @cells.max{|c1, c2| c1.ideal_height <=> c2.ideal_height}.ideal_height 
      end
      @height = @ideal_height
    end
    @height ||= @@DEF_HEIGHT
  end

  def to_s
    '#<' << self.class.name << ':' << object_id.to_s << "{number=%s height=%s cells.length=%s}" %[@number.to_s, @height.to_s, @cells.length.to_s] 
  end

  attr_reader :number, :height, :cells

  alias :<< :add

end

