#encoding: UTF-8

=begin
/***************************************************************************
 *   Copyright ©2017-2017, Michael Uplawski <michael.uplawski@uplawski.eu> *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
=end
if __FILE__ == $0
  require_relative 'user_input'
end

require_relative 'logging'
require_relative 'color_output'

class ScrollException < StandardError
end

# Objects which include this module are supposed to be multi-line text as they
# have to respond to :to_s.  The new functionality will allow this text to
# become scrollable in a terminal-window. See the scroll-functions up(),
# down(), left() and right().
#
# You can have a box around the scrollable "window" by setting the attribute
# 'box' to true and, -as I do not care to be asked for my opinion-, you should.
module Scrollable
  self.extend(Logging)
  # may not be needed at all, but when it is..,
  # you want it on class level.
  @@box = %w~┌ ┐ └ ┘ ─ │~

  # dto. and how so!
  @@log = self.init_logger(STDOUT)

  # pfffft. As you please.
  @@def_step_x = 1
  @@def_step_y = 1

  # does it
  def show
    defaults() if !@have_defaults
    system('clear')
    new_view = limit_lines
    if @box  
      box(new_view)
    else
      puts new_view
    end
  end

  # guess
  def scroll_x(offset)
    defaults(self) if !@have_defaults
    if(offset != 0)
      xt = @xoff + offset
      width_x = num_columns - xt
      if(width_x > 0)
        @xoff = xt
      end
    end
    show()
  end

  # scroll up or down by offset lines.
  def scroll_y(offset)
    defaults() if !@have_defaults
    # any offset is okay
    if(offset != 0 )
      yt = @yoff + offset
      height_y = @lines.length + yt
      if(height_y > 0)
        @yoff = yt
      end
    end
    show()
  end

  # scrolls
  def left()
    defaults() if !@have_defaults
    scroll_x(@step_x * -1)
  end

  # dto. To the right. You guessed that.
  def right()
    defaults() if !@have_defaults
    scroll_x(@step_x)
  end

  # scrolls
  def up()
    defaults() if !@have_defaults
    scroll_y(@step_y * -1 )
  end

  # scrolls
  def down
    defaults() if !@have_defaults
    # increase number of visible lines by step_y
    scroll_y(@step_y)
  end

  # scrolls
  def scroll(xoffset, yoffset)
    defaults() if !@have_defaults
    scroll_y(yoffset)
    scroll_x(xoffset)
  end

  # read the code
  def home() 
    first_line
    first_column
  end

  # dto.
  def to_end()
    last_column
    last_line
  end

  def first_line()
    defaults() if !@have_defaults
    @yoff = 0
  end

  def last_line()
    defaults() if !@have_defaults
    @yoff = @lines.length 
  end

  def first_column()
    defaults() if !@have_defaults
    @xoff = 0
  end

  def last_column()
    defaults() if !@have_defaults
    @xoff = num_columns - 1
  end
  # set the size of a vertical scroll step
  def step_y=(step)
    if step > 0
      @step_y = step
    else
      raise ScrollException.new('ERROR: scroll-steps must be positive numbers')
    end
  end

  # set the size of a horizontal scroll step
  def step_x=(step)
    if step > 0
      @step_x = step
    else
      raise ScrollException.new('ERROR: scroll-steps must be positive numbers')
    end
  end

  def width=(w)
    @width = w if w >= 1
  end

  def height=(h)
    @height = h if h >= 1
  end

  attr_accessor :fixed_cols, :fixed_rows
  attr_writer :box
  attr_reader :width, :height, :xoff, :yoff, :clipped_bottom, :clipped_top, :clipped_left, :clipped_right

  private

  # Clip off all that cannot fit into the defined width and height.  Removes
  # probably more than needed, but in its current state, I tend to *understand*
  # this method. In addition, it somehow appears to work for me...
  def limit_lines()
    # limit lines laterally
    if(@xoff < 0)
      @xoff = 0
    end
    new_view = @lines.collect do |l| 
      short = l[0...@fixed_cols].to_s + l[(@xoff + @fixed_cols)..(@width + @xoff)].to_s
    end


    # limit number of lines
    if(@yoff >= 0)
      if(@yoff > (@lines.length - @height + @fixed_rows))
        @yoff = @lines.length - @height + @fixed_rows
      end
    end

    @yoff = 0 if @yoff < 0
    new_view = new_view[0...@fixed_rows] + new_view[(@yoff + @fixed_rows)...(@height + @yoff - @fixed_rows) ]

    # describe the situation in case that someone must know..,
    if @box
      @clipped_left = @xoff > 0
      @clipped_top = @yoff > 0
      @clipped_right = num_columns - @xoff > @width
      @clipped_bottom = @lines.length - @yoff > @height
    end
    #... not, if not

    return new_view.join("\n")
  end

  # calculates the number of columns (characters), needed.
  def num_columns
    defaults if !@have_defaults
    @lines.max {|a, b| a.length <=> b.length}.length
  end

  # sets a few values, needed for the rest of the module to function.
  def defaults()
    @log = @@log
    if(self.respond_to?(:to_s) )
      # @view = self
      @view = self.gsub(/\e\[\d+m/, '')

      @lines = @view.split("\n")

      @xoff = 0
      @yoff = 0

      @step_x ||= @@def_step_x
      @step_y ||= @@def_step_y

      @width ||= 60
      @height ||= 30

      @fixed_rows ||= 0
      @fixed_cols ||= 0

      @box ||= false

      @have_defaults = true 
    elsif !view
      msg = 'Need to know, what to scroll! (argument "view" missing)'
      @log.error(yellow(msg))
      raise ScrollException.new(msg)
    else
      msg = 'The "view" to scroll must have a method "to_s()"'
      @log.error(yellow(msg))
      raise ScrollException.new(msg)
    end
  end

  # draw lines of color, wherever the content is clipped.

  def box(view)
    nl = "\n"
    lines = view.split(nl)

    ctl = @@box[0]
    bl = @@box[5]
    cbl = @@box[2]

    bt = @@box[4]

    ctr = @@box[1]
    br = @@box[5]
    cbr = @@box[3]

    bb = @@box[4]

    if clipped_left
      ctl = bold(blue(ctl))
      bl = bold(blue(bl))
      cbl = bold(blue(cbl))
    end
    if clipped_right
      ctr = bold(blue(ctr))
      br = bold(blue(br))
      cbr = bold(blue(cbr))
    end

    if clipped_top
      ctr = bold(blue(ctr))
      bt = bold(blue(bt))
      ctl = bold(blue(ctl))
    end

    if clipped_bottom
      cbl = bold(blue(cbl))
      cbr = bold(blue(cbr))
      bb = bold(blue(bb))
    end
    size = `stty size`.split.map { |x| x.to_i }.reverse
    @width -= 1 until (@width + 5) <= size[0]
    #@height -= 1 until (@height + 2) <=  size[1]
    puts "%s%s%s" %[ctl, bt * (@width + 3), ctr ]
    height.times {|n| puts "%s %-#{@width+1}s %s" %[bl, lines[n], br] }
    puts "%s%s%s" %[cbl, bb * (@width + 3), cbr ]
  end
end

#### TEST

if __FILE__ == $0
  # ------------> create a view
  h = 60
  w = 30
  test_view = "   |"
  w.times {|col| test_view << "%s%2d|" %['c', col.next]} 
  test_view << "\n" << "-" * test_view.length << "\n"
  h.times do |row|
    test_view << "r%2d|" %row.next 
    w.times do |col| 
      content = "%3d" %col.next
      if( col%3 == 0)
        content = "\e[33m" << content << "\e[0m" 
      end
      test_view << "%s|" %content
    end
    test_view << "\n"
  end
  # <------------- View done

  test_view.extend(Scrollable)

  # show all
  test_view.show
  while true 
    case wait_for_user.chr
      # scroll
    when "i"
      test_view.up
    when "k"
      test_view.down
    when "j"
      test_view.left
    when "l"
      test_view.right
      # change step-width
    when "y"
      print "step y: "
      step = wait_for_user.chr().to_i
      puts step
      begin
        test_view.step_y = step
        puts "step y changed\n" 
      rescue Exception => ex
        puts ex.message
      end
      test_view.show
    when "r"
      print "fix rows: " 
      fr = wait_for_user.chr().to_i
      begin
        test_view.fixed_rows = fr
        puts "fixed " << fr.to_s << "rows"
      rescue Exception => ex
        puts ex.message
      end
      test_view.show
    when "c"
      print "fix columns: "
      fc = wait_for_user.chr().to_i
      begin
        test_view.fixed_cols = fc
        puts "fixed " << fc.to_s << "columns"
      rescue Exception => ex
        puts ex.message
      end
      test_view.show
    when 'w'
      print "display width: "
      w = STDIN.gets.chomp.to_i
      test_view.width = w
      test_view.show
    when 'q'
      exit true
    end
  end
end


