#encoding: UTF-8

=begin
/*****************************************************************************
 *   Copyright © 2016-2016, Michael Uplawski <michael.uplawski@uplawski.eu>  *
 *                                                                           *
 *   This program is free software; you can redistribute it and/or modify    *
 *   it under the terms of the GNU General Public License as published by    *
 *   the Free Software Foundation; either version 3 of the License, or       *
 *   (at your option) any later version.                                     *
 *                                                                           *
 *   This program is distributed in the hope that it will be useful,         *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 *   GNU General Public License for more details.                            *
 *                                                                           *
 *   You should have received a copy of the GNU General Public License       *
 *   along with this program; if not, write to the                           *
 *   Free Software Foundation, Inc.,                                         *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.               *
 *****************************************************************************/
=end

require 'roo'
require 'roo-xls'
require 'filemagic'

require_relative 'logging'

# Transforms a file into a Roo spreadsheet instance.

ODS_Magic = "OpenDocument Spreadsheet"
XLS_Magic = "Composite Document File V2 Document"
XLSX_Magic = "Microsoft OOXML"

ODS_MIME = "application/vnd.oasis.opendocument.spreadsheet"
XLS_MIME = "application/vnd.ms-excel"
XLSX_MIME = "application/octet-stream"
CSV_MIME = "text/plain"

class SheetData
  self::extend(Logging)
  @@log = init_logger(STDOUT, Logger::INFO)

  @@file = nil		
  @@workbook = nil

  def self::workbook(file)
    @@file = file
    if !@@workbook
      begin
        magic, mime = file_type
        @@workbook = case mime
                     when ODS_MIME 
                       if magic.match ODS_Magic
                         @@log.debug('ODS')
                         Roo::Spreadsheet::open(@@file, extension: :ods ) 
                       end
                     when XLS_MIME 
                       if magic.match XLS_Magic
                         @@log.debug('XLS')
                         Roo::Spreadsheet::open(@@file, extension: :xls ) 
                       end
                     when XLSX_MIME 
                       if magic.match XLSX_Magic
                         @@log.debug('XLSX')
                         Roo::Spreadsheet::open(@@file, extension: :xlsx ) 
                       end
                     when CSV_MIME
                       raise IOError.new('CSV is not yet supported, sorry')
                     else
                       raise IOError.new('Mime-Type is ' << mime << ' and Magic sais: ' << magic << ". Is this supposed to be a spreadsheet?")
                     end
      rescue Exception => ex
        msg = 'ERROR! File %s is not supported: %s' %[@@file, ex.message]  
        puts msg
        @log.error yellow(msg)
        exit false
      end			

      return @@workbook
    end
  end
 
  private
  # derive the mime-type from the file
  def self::file_type
    fm = FileMagic.fm
    file_magic = fm.file(@@file)
    fm.flags = [:mime_type]
    file_mime = fm.file(@@file)	
    @log.debug('File type is ' << file_magic << ', Mime-type is ' << file_mime)
    return file_magic, file_mime
  end
end

