#!/usr/bin/ruby
#encoding: UTF-8

=begin
/******************************************************************************
 *   Copyright © 2014-2016, Michael Uplawski <michael.uplawski@uplawski.eu>   *
 *                                                                            *
 *   This program is free software; you can redistribute it and/or modify     *
 *   it under the terms of the GNU General Public License as published by     *
 *   the Free Software Foundation; either version 3 of the License, or        *
 *   (at your option) any later version.                                      *
 *                                                                            *
 *   This program is distributed in the hope that it will be useful,          *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of           *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            *
 *   GNU General Public License for more details.                             *
 *                                                                            *
 *   You should have received a copy of the GNU General Public License        *
 *   along with this program; if not, write to the                            *
 *   Free Software Foundation, Inc.,                                          *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.                *
 ******************************************************************************/
=end

require_relative 'logging'
require_relative 'file_checking'
require_relative 'sheetinterface'
require_relative 'sheetdata'
require_relative 'color_output'

require 'roo'
require 'roo-xls'
require 'tempfile'

class ViewWorkBook
  include Logging
  include File_Checking

  attr_reader :workbook

  def initialize(*args)
    init_logger(STDOUT, Logger::INFO)
    @log.debug('args is ' << args.to_s)
    wb = args[0]
    path = File.path(wb) if wb
    @log.debug('workbook file will be ' << (path ? path : 'N I L'))

    if(wb)
      msg = file_check(path, :exist?, :file?, :readable?)
      if(!msg)
        begin
          @workbook = SheetData.workbook(path)
          view_sheet if @workbook
        end
      else
        @log.error(yellow("Cannot open " << path << ": " << msg))
        # raise IOError.new(msg)
        puts red("\n\tPlease name a valid spreadsheet file! Aborting.")
        puts
        exit false
      end
    end
    @log.debug('initialized')
  end

  def method_missing(method, *args)
    @workbook.send(method.to_sym, *args)
  end

  def sheet(number)
    view_sheet(number)
  end

  private
  def view_sheet(number = 0)
    @log.debug('default_sheet is ' << sheets[number]) 
    SheetInterface.new(@workbook, number)
  end
end

if __FILE__ == $0
  ViewWorkBook.new(ARGV)	
end
